#include <stdio.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#ifdef HAVE_SYS_AUDIOIO_H
#include <sys/audioio.h>
#endif /* HAVE_SYS_AUDIOIO_H */
#ifdef HAVE_LINUX_SOUNDCARD_H
#include <linux/soundcard.h>

/* Older linux/soundcard.h doesn't define AFMT_S16_NE */
#ifndef AFMT_S16_NE
#ifdef WORDS_BIGENDIAN
#define AFMT_S16_NE AFMT_S16_BE
#else
#define AFMT_S16_NE AFMT_S16_LE
#endif /* WORDS_BIGENDIAN */
#endif /* AFMT_S16_NE */

#endif /* HAVE_LINUX_SOUNDCARD_H */
#ifdef USE_ESD
#include <esd.h>
#endif

#include "psg.h"

static int audio_dev;

void open_audio(PSG_Settings *set)
{
  int fd = -1;

  if (set->output)
    {
      if (strcmp (set->output, "-") == 0)
	fd = 1;
      else
	fd = open (set->output, O_WRONLY | O_CREAT, 0666);
    }
  else
    {
#ifdef USE_ESD
      int rate = ESD_DEFAULT_RATE;
      int bits = ESD_BITS16, channels = ESD_STEREO;
      int mode = ESD_STREAM, func = ESD_PLAY ;
      esd_format_t format = 0;

      format = bits | channels | mode | func;

      fd = esd_play_stream( format, rate, "localhost", "tjo" );
#elif HAVE_SYS_AUDIOIO_H
      char *devname;
      audio_info_t audio_info;

      devname = getenv("AUDIODEV");
      if(devname == NULL)
	devname = "/dev/audio";
  
      fd = open(devname, O_WRONLY);
  
      ioctl(fd, AUDIO_GETINFO, &audio_info);
      audio_info.play.sample_rate = 44100;
      audio_info.play.channels = 2;
      audio_info.play.precision = 16;
      audio_info.play.encoding = AUDIO_ENCODING_LINEAR;
      ioctl(fd, AUDIO_SETINFO, &audio_info);
#elif HAVE_LINUX_SOUNDCARD_H
      int format = AFMT_S16_NE;
      int speed = 44100;
      int stereo = 1;

      fd = open("/dev/dsp", O_WRONLY);

      ioctl(fd, SNDCTL_DSP_RESET, 0);
      ioctl(fd, SNDCTL_DSP_SETFMT, &format);
      ioctl(fd, SNDCTL_DSP_SPEED, &speed);
      ioctl(fd, SNDCTL_DSP_STEREO, &stereo);
#else
#error Could not find any suitable way to use audio on your machine
#endif /* Audio checks */
    }

  if(fd < 0) {
    fprintf(stderr, "Error opening audio.\n");
    exit(-33);
  }

  audio_dev = fd;
}

void close_audio(void)
{
  close(audio_dev);
}

void play_sample(signed short *buffer, int size)
{
  write(audio_dev, buffer, size);
}
