#ifndef PSG_H
#define PSG_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef FIXED_MATH
#  define FIXED_TYPE int
#  define FIXED(x) ((int)((x) * 256))
#  define DIV(x, y) ((((x) + ((y) >> 1)) << 8) / (y))
#  define MOD(x, y) ((x) % (y))
#  define FIXED_TO_INT(x) (((x) + 128) >> 8)
#else
#  define FIXED_TYPE double
#  define FIXED(x) x
#  define DIV(x, y) ((x) / (y))
#  define MOD(x, y) fmod (x, y)
#  define FIXED_TO_INT(x) (x)
#endif

#define MAX_CHANNELS 3
#define MFP_FREQUENCY 2457600
#define PSG_FREQUENCY 2000000
#define SAMPLE_FREQ 44100
#define SAMPLE_CHAN 2
#define SAMPLE_BITS 16
#define SAMPLE_OFFSET (SAMPLE_CHAN*SAMPLE_BITS/8)
#define SAMPLE_BUFFER (SAMPLE_FREQ*SAMPLE_CHAN*SAMPLE_BITS/8)
#define VBL 50
#define DETAIL (1<<14)
#define DETAIL_LOOP (DETAIL-1)
#define TYPES 3
#define ECHO_DISTANCE 5
#define ECHO_LEVEL 7
#define INLINE static __inline__

typedef struct {
  unsigned char *buffer;
  int size;
} raw_buffer;

typedef struct {
  int volume;
  int frequency;
  int psg_freq;
  int env_enable;
  int tone_enable;
  int timer_enable;
  int noise_enable;
  int period_count;
} Channel;

typedef struct {
  Channel channel[MAX_CHANNELS];
  int env_mode;
  int env_psg_freq;
  FIXED_TYPE env_frequency;
  int env_period_count;
  int env_period_hold;
  int timer_freq;
  int timer_duration;
  int timer_count;
  int timer_vmax;
} VBL_Frame;

typedef struct {
  long id;
  long length;
  long mask;
  int regs;
  short no_digi;
  char *name;
  char *name2;
  char *name3;
  signed short tone[DETAIL][TYPES];
  signed short noise[DETAIL];
  unsigned char *data;
} YM_Header;

typedef struct {
  int echo;
  int verbose;
  int different;
  int repeat;
  int quiet;
  int stereo;
  int surround;
  char *output;
} PSG_Settings;

#ifdef WORDS_BIGENDIAN
#define SWAPSHORT(x) (x)
#define SWAPLONG(x) (x)
#define SWAPSTOLITTLE(x) (((x & 0xff00)>>8)|((x & 0xff)<<8))
#define SWAPLTOLITTLE(x) (((x & 0xff000000)>>24)|((x & 0xff0000)>>8)|\
                          ((x & 0xff00)<<8)|((x & 0xff)<<24))
#else
#define SWAPSHORT(x) (((x & 0xff00)>>8)|((x & 0xff)<<8))
#define SWAPLONG(x) (((x & 0xff000000)>>24)|((x & 0xff0000)>>8)|\
                     ((x & 0xff00)<<8)|((x & 0xff)<<24))
#define SWAPSTOLITTLE(x) (x)
#define SWAPLTOLITTLE(x) (x)
#endif

#ifndef HAVE_GETOPT_LONG
#define getopt_long(a,b,c,d,e) getopt(a,b,c)
#endif

raw_buffer *load_file(char *);
void open_audio(PSG_Settings *);
YM_Header *psg_build_header(unsigned char *, int, PSG_Settings *);
void psg_play(YM_Header *, PSG_Settings *);
void play_sample(signed short *, int);
FIXED_TYPE env_wave(FIXED_TYPE, int);

#endif /* PSG_H */
